module DallasMarsRover
  # The main Mars Rover class
  class MarsRover
    # @return [String] the resulting rover location
    def start
      # Validate we have only files in the argument list
      abort('ABORTED! Supply a command file') unless valid_arguments?
      # Exit if we cant create the main processor
      abort('ABORTED! Invalid commands in file') unless create_commander
      # Create mars zone
      create_zone
      # Create mars rover
      create_rover
      # Everything ready, Lets drive!
      start_driving
      # Display end location
      display_location
    end

    private

    attr_reader :commander, :zone, :rover, :file

    # @return [@commander] main app processor object
    def create_commander
      @commander ||= Commander.new @file
    end

    # @return [@zone] the survey zone object
    def create_zone
      @zone ||= Zone.new @commander
    end

    # @return [@rover] the mars rover vehicle object
    def create_rover
      @rover ||= Rover.new @commander, @zone
    end

    # @return [Boolean] drive status operation
    def start_driving
      rover.drive
    end

    # @return [String] shows the end result co-ordinates
    def display_location
      puts "#{rover.pos_x} #{rover.pos_y} #{rover.direction}"
    end

    # @return [Boolean] if arguments are valid
    def valid_arguments?
      # Loop through the list, removing anything that is not a valid file
      ARGV.each do |filename|
        ARGV.delete(filename) unless File.file?(filename)
      end
      # After processing, anything left? and only one input file
      return false unless ARGV.length == 1
      @file = File.open ARGV.join
    end
  end
end
