module DallasMarsRover
  # The "physical" rover class
  class Rover
    attr_reader :pos_x, :pos_y, :direction
    # Load start position and direction
    # @param [Commander] commander object
    # @param [Zone] zone object
    # @return [Rover] rover object
    def initialize(commander, zone)
      @commander = commander
      @zone = zone
      @pos_x = commander.start_point[:x].to_i
      @pos_y = commander.start_point[:y].to_i
      @direction = commander.start_direction
    end

    # This is the main loop for processing the commands
    # @return [Boolean]
    def drive
      commander.commands.each do |command|
        case command
          when 'L'
          new_index = (COMPASS.index(@direction) - 1).negative? ? (COMPASS.count - 1) : COMPASS.index(@direction) - 1
          @direction = COMPASS[new_index]
        when 'R'
          new_index = (COMPASS.index(@direction) + 1) > (COMPASS.count - 1) ? 0 : COMPASS.index(@direction) + 1
          @direction = COMPASS[new_index]
        else
          abort('Aborted! Rover moved out of zone') unless move
        end
      end
    end

    private

    attr_reader :commander, :zone

    # @return [Boolean] whether move was successful
    def move
      case direction
      when 'N'
        # We move up y-axis, +y
        return false unless zone.valid_position?(pos_x, pos_y + 1)
        @pos_y += 1
      when 'E'
        # We move right on x-axis, +x
        return false unless zone.valid_position?(pos_x + 1, pos_y)
        @pos_x += 1
      when 'S'
        # We move down y-axis, -y
        return false unless zone.valid_position?(pos_x, pos_y - 1)
        @pos_y -= 1
      when 'W'
        # We move left on x-axis, -x
        return false unless zone.valid_position?(pos_x - 1, pos_y)
        @pos_x -= 1
      end
    end
  end
end
